% :- ensure_loaded( '../src/lib/els'  ).
% :- els( '../src/init_lib'  ).
:- ensure_loaded( '../src/init_lib'  ).
:- ensure_loaded( '../src/all_dynamic'  ).
:- ensure_loaded( '../src/ad_to_slp'  ).
:- ensure_loaded( 'eps_n_latex_graph_freqs' ).		%.

:- pl( swi(_), true, ensure_loaded(library(flatten)) ).	     % /2.
:- ensure_loaded( library(slp_in_options) ).	% /2.
:- ensure_loaded( library(tmp_in_options) ).	% /3.
:- ensure_loaded( library(pl_to_slp_call) ).	% _args/3.
:- ensure_loaded( library(list_frequency) ).	% /2. 
:- ensure_loaded( library(write_list_elem_per_line_w_period) ).	% /1.
:- ensure_loaded( library(kv_fold_v_in_k_term) ).				% /1.
:- ensure_loaded( library(kv_decompose) ).					% /3.
:- ensure_loaded( library(to_list) ).						% /2.

:- ensure_loaded( library(lists) ).		% append/3, memberchk/2.
:- ensure_loaded( library(ordsets) ).		% list_to_ord_set/2
:- ensure_loaded( library(charsio) ).		% write_to_chars/2.
:- ensure_loaded( library(nth_replace) ).		% /5.

% probably we need to insist real model space is used...

% Opts has, slp/1, call/1, tmp/1, stem/1.
all_models_by_failure( Opts ) :-
	memberchk( help, Opts ),
	!,
	nl, nl,
	write( 'all_models,  script for reporting all models generated by an SLP.' ),
	nl, nl, write( 'Options:' ), nl, nl,
	write( '   slp/1              SLP file to load.' ), nl,
	write( '   stem/1             stem for output file.' ), nl,
	write( '   call/2             call(Call, Obs). The call and its observable.' ), nl,
	write( '   latex/1            {true,*false*}. Report on a latex file.' ), nl,
	write( '   lhood/1            {true,norm,*false*}' ), nl,
	write( '   post/1             translate call, invoked after each success. Default is "true".' ), nl,
	write( '   pre/1              call to invoked before findall. Default is "true".' ), nl,
	write( '   rec/1              predicate name and selectors for constructing recording item. By default Obs is recorded, with obs/1 name. Selectors a list of: [lhood,obs,path,prb], or "all"' ), nl,
	write( '   terms/1            boolean (true), record observations as terms.' ), nl,
	write( '   to_frq/1           boolean (true) or last, gather to Recorded-Tms pairs before latex (Tms last in recorded term when "last" is given)' ), nl, nl.
	% efs = [stem('split-2-all'),
			% % call(cart(NofLs),NofLs),
			% slp('split.slp'),
			% pre(true),
			% post(true),
			% rec(rec_def(obs)),
			% to_frq(true)].

% pupsh all_models slp=../compare/true/asia.slp call=sample=lst=A,S,T,L,B,E,D,X::lst=A,S,T,L,B,E,D,X rec=obs=lst=obs,prb

all_models_by_failure( Opts ) :-
	( memberchk(stem(Stem),Opts) ->
		true
		;
		Stem = observations
	),
	atom_concat( Stem, '.obs', ObsFile ),
	tell( ObsFile ),
	all_models( Opts, Freqs, Tmp, Out, GoalTrmCs, ObsCs, Slp, NofMs ),
	told, write( ok  ), nl, halt(0),
	% flatten( [" finding all solutions of \\verb?",GoalTrmCs," observing ",ObsCs,"?."], ForCs ),
	( memberchk(latex(true),Opts) -> 
		number_codes( NofMs, NMsCs ),
		flatten( [" finding all solutions of \\verb?",GoalTrmCs," observing ",ObsCs,".", NMsCs, " models found.?"], ForCs ),
		atom_codes( For, ForCs ),
		eps_n_latex_graph_freqs( Freqs, Out, For, 1, Slp, Tmp )
		;
		true
	),
	( memberchk(terms(false),Opts) -> 
		true
		;
		( memberchk(stem(Stem),Opts) ->
			true
			;
			Stem = observations
		),
		atom_concat( Stem, '.obs', ObsFile ),
		tell( ObsFile ),
		write_list_elem_per_line_w_period( Freqs ),
		told
	),
	delete_file_if( Tmp ).

all_models_by_failure( _Opts ) :-
	told, write( ok  ), nl, halt(0).

all_models( Opts, Freqs ) :-
	all_models( Opts, Freqs, Tmp, _Out, _GoalTrmCs, _ObsCs, _Slp ),
	delete_file_if( Tmp ).

all_models( Opts, Freqs, Tmp, Out, GoalTrmCs, ObsCs, Slp, NofMs ) :-
	all_models_defaults( Defs ),
	append( Opts, Defs, All ),
	slp_in_options( All, Slp ),
	tmp_in_options( Opts, All, Tmp ),
	all_dynamic( Slp ),
	ad_to_slp( [rm(false),tmp(Tmp)] ),
	memberchk( call(PlCall,Obs), All ),
	pl_to_slp_call_args( PlCall, SlpCall, CallArgs ),
	memberchk( path(B), CallArgs ),
	memberchk( pre(Pre), All ),
	call( Pre ),
	( (memberchk(lhood(Lhood),All),Lhood\==false) -> 
		LhCall = model_llhood( Obs, Lh )
		;
		Lhood = false, LhCall = true
	), % HERE.
	memberchk( post(Post), All ),
	memberchk( rec(RecOpt), All ),
	record_this( RecOpt, B/_Bp, Prb/_Pp, Obs/Op, Lh/Lp, _RecCall, Rec ),
	%% findall( Rec,
	% findall( (B,Obs)-Prb, 
	% findall( (B,Prb,Obs)-1, 
		%% (
               call( slp:SlpCall ),
			B = Path/[],
			% write( path(Path) ), nl,
			path_to_prb(Path,1,Prb),
               write( Rec ), nl,
               fail,
			call(Post),
			% LhCall
			% call( RecCall )
			%% )
				% call(Post), write( called(Obs-Prb) ), nl)
			%% , Found ),

	( memberchk(out(Out),Opts) -> true ; memberchk(stem(Out),All) ),
	length( Found, NofMs ),
	( Lhood == norm -> term_on_nth_pbc_normalisation( Found, Lp, LhNrmFound )
		; LhNrmFound = Found ),
	( (memberchk(to_frq(ToFrq),Opts),ToFrq\==false) -> 
		term_on_nth_frequency_pairs( LhNrmFound, Op, FrqPrs ),
		( ToFrq == last ->
			kv_fold_v_in_k_term( FrqPrs, Freqs )
			;
			Freqs = FrqPrs
		)
		; 
		LhNrmFound = Freqs
	),
	write_to_chars( PlCall, GoalTrmCs ),
	write_to_chars( Obs, ObsCs ).

record_this( Rec, B, Prb, Obs, Lh, _RecCall, RecThis ) :-
	Rec =.. [Name,PrvSelectors],
	to_list( PrvSelectors, Selectors ),
	% list_to_ord_set( Selectors, OrdSels ),
	remove_duplicates( Selectors, UnqSels ),
	record_this_selectors( UnqSels, 1, B, Prb, Obs, Lh, Args ),
	% append( Args, [RecStr], AArgs ),
	% RecCall =.. [Name|AArgs].
	RecThis =.. [Name|Args].

record_this_selectors( all, _One, B/2, Prb/3, Obs/1, Lh/4, List ) :-
	List = [Obs,B,Prb,Lh].
record_this_selectors( [], _I, _B, _Prb, _Obs, _Lh, [] ).
record_this_selectors( [H|T], I, B, Prb, Obs, Lh, Args ) :-
	( H == obs -> Obs = ObsTrm/I, Args = [ObsTrm|Targs]
		; ( H == path -> B = BTrm/I, Args = [BTrm|Targs]
			; ( H == prb -> Prb = PTrm/I, Args = [PTrm|Targs] 
				; ( H == lhood -> Lh = LTrm/I, Args = [LTrm|Targs]
					; write( user_error, 'Skipping unknown record id: ' ), 
				  	write( user_error, H ), nl( user_error ),
				  	write( user_error, 'Recognised are: lhood, obs, path, prb.' ),
				  	nl( user_error ),
				  	Targs = Args ) ) ) ),
	NxI is I + 1,
	record_this_selectors( T, NxI, B, Prb, Obs, Lh, Targs ).


rec_def( Any, Any ).
rec_def( First, Second, First/Second ).
rec_def( First, Second, Third, First:Second/Third ).

all_models_defaults( Defs ) :-
	Defs = [stem('split-2-all'),
			call(cart(NofLs)-NofLs),
			slp('split.slp'),
			post(true),
			pre(true),
			rec(obs([obs])),
			to_frq(true)].

path_to_prb( [], Prb, Prb ).
path_to_prb( [H|T], Acc, Prb ) :-
	( is_list(H) -> 
		path_to_prb( H, Acc, MdAcc ),
		path_to_prb( T, MdAcc, NxAcc )
		% write( user_error, nested ), nl( user_error ), abort
		; (H = (_Id1/HPrb) ->
			NxAcc is HPrb * Acc
			;
			( H = (_Id2:HPrb) ->
				NxAcc is HPrb * Acc
				;
				NxAcc is Acc
			)
		   )
	),
	path_to_prb( T, NxAcc, Prb ).

term_on_nth_pbc_normalisation( List, Nth, Norm ) :-
	term_on_nth_summation( List, Nth, 0, Sum ),
	term_on_nth_div_by( List, Nth, Sum, Norm ). 

term_on_nth_summation( [], _Nth, Sum, Sum ).
term_on_nth_summation( [H|T], Nth, Acc, Sum ) :-
	arg( Nth, H, Arg ), 
	NxAcc is Acc + Arg,
	term_on_nth_summation( T, Nth, NxAcc, Sum ).

term_on_nth_div_by( [], _Nth, _Dnm, [] ). 
term_on_nth_div_by( [H|T], Nth, Dnm, [DivH|DivT] ) :-
	H =.. [Name|Args],
	nth_replace( Nth, Args, NthDiv, NthArg, DivArgs ),
	NthDiv is NthArg / Dnm,
	DivH =.. [Name|DivArgs],
	term_on_nth_div_by( T, Nth, Dnm, DivT ). 

term_on_nth_frequency_pairs( LhNrmFound, Nth, FrqPrs ) :-
	term_on_nth_frequency_pairs_1( LhNrmFound, Nth, [], FrqTrips ),
	kv_decompose( FrqTrips, _, FrqPrs ).

term_on_nth_frequency_pairs_1( [], _Nth, Trips, Trips ).
term_on_nth_frequency_pairs_1( [H|T], Nth, Acc, Trips ) :-
	arg( Nth, H, Arg ),
	( select(Arg-(SeenTerms-SeenTms),Acc,RmAcc) -> 
		cohese_args_except_nth( SeenTerms, H, Nth, NxSeen ),
		NxTms is SeenTms + 1,
		NxAcc = [Arg-(NxSeen-NxTms)|RmAcc]
		;
		NxAcc = [Arg-(H-1)|Acc]
	),
	term_on_nth_frequency_pairs_1( T, Nth, NxAcc, Trips ).

cohese_args_except_nth( Seen, New, Nth, Next ) :-
	Seen =.. [Name|SeenArgs],
	New  =.. [Name|NewArgs],
	cohese_lists_except_nth( SeenArgs, NewArgs, 1, Nth, NextArgs ),
	Next =.. [Name|NextArgs].

cohese_lists_except_nth( [], Sec, _I, _Mark, Nxt ) :-
	Sec == [],
	Nxt = [].
cohese_lists_except_nth( [H|T], [F|R], I, Mark, [ChH|ChT] ) :-
	( I =:= Mark ->
		ChH = H
		;
		( is_list(H) -> ChH = [F|H] ; ChH = [F,H] )
	),
	NxI is I + 1,
	cohese_lists_except_nth( T, R, NxI, Mark, ChT ).
