
:- ensure_loaded( library(lists) ).				% memberchk/2.
:- ensure_loaded( '../src/init_lib' ).

:- requires( opts_have_help/1 ).
:- requires( opts_select_infiles ).
:- requires( file_to_list_of_lines/2 ).
:- requires( read_terms/2 ).

% upsh ~/islp/mcmcms/auxil/diff_ratio *ratio

diff_base( Opts ) :-
	opts_have_help( Opts ),
	!,
	write( 'diff_base:         report line differences from a base file.' ),
	nl, nl,
	write( 'Options.' ), nl,
	write( '  base(B)          for the base file' ),
	write( '  file(F)          for the file. An Atom is also acceptable' ),
	nl.    

diff_base( Opts ) :-
	opts_select_infiles( Opts, Fs, _Oth ),
	( Fs = [Fst|T] -> 
		( T == [] ->
			true
			;
			write( user_error, more_than_one_file_present-ignoring(T) ),
			nl( user_error )
		),
		( memberchk( base(Base), Opts ) ->
			diff_from_base( Base, Fst )
			;
			write( user_error, couldnt_find_base_in_options ), 
			nl( user_error )
		)
		;
		write( user_error, at_least_one_file_should_be_given(Opts) ),
		nl( user_error )
	).

diff_from_base( Base, Fst ) :-
	file_to_list_of_lines( Base, BaseLns ),
	read_terms( Fst, FstTerms ),
	diff_from_base_lines( BaseLns, FstTerms, 0 ).

diff_from_base_lines( [], Terms, KL ) :-
	( Terms == [] -> 
		true
		; 
		length( Terms, ExLength ),
		write( user_error, there_are_extra_lines_at_end_of_second_file(ExLength) ),
		nl( user_error ), nl( user_error )
	),
	write( kl(KL) ), nl.

diff_from_base_lines( [Hbase|Tbase], Terms, KLAcc ) :-
	( Terms = [Class:Nom/Dnm|TTerms] ->
		number_codes( BaseClass, Hbase ),
		( Class == BaseClass ->
			Diff is 1 - (Nom/Dnm),
			NxKLAcc is KLAcc + log((Dnm/Nom))
			;
			Diff is Nom / Dnm,
			NxKLAcc is KLAcc + log((1/(1 - (Nom/Dnm))) ),
			write( NxKLAcc is KLAcc + log((1/(1 - (Nom/Dnm))) ) ), nl
		),
		write( Diff ), nl,
		RstB = Tbase,
		write( then-bc(BaseClass)-cl(Class) ), nl
		;
		write( bc(BaseClass)-cl(Class) ), nl,
		length( [Hbase|Tbase], ExLength ),
		write( user_error, there_are_extra_lines_at_end_of_first_file(ExLength) ),
		nl( user_error ), nl( user_error ),
		RstB = [],
		TTerms = [],
		NxKLAcc is KLAcc
	),
	write( klacc(KLAcc) ), nl,
	diff_from_base_lines( RstB, TTerms, NxKLAcc ).
