
:- ensure_loaded( '../src/init_lib' ).

:- ensure_loaded( mvis_to_fterms ).
:- ensure_loaded( library(gzip_easy) ).			% /1.
:- ensure_loaded( library(gunzip_easy) ).		% /1.
:- ensure_loaded( library(mcmcms_top_dir) ).		% bb_puts it
:- ensure_loaded( library(elm) ).				% /1.

fterms( [H|T] ) :-
	bb_get( mcmcms_top_dir, Dir ),
	load_data( H, Dir, Type ),
	fterms_1( T, Type ).

fterms_1( [], _Type ).
fterms_1( [H|T], Type ) :-
	gunzip_easy( H, HStem ),
	( atom_concat( Pfx, '.mvisits', HStem ) ->
		HPl = H ; Pfx = HStem, atom_concat(HStem,'.mvisits',HPl) ),
	atom_concat( Pfx, '.fterms', FPl ),
	mvis_to_fterms( HPl, Type, FPl ),
	gzip_easy( H ),
	fterms_1( T, Type ).

load_data( ped_exs, Dir, ped ) :-
	!,
	atom_concat( Dir, 'pedigree/data/exs', File ),
	ensure_loaded( File ).
load_data( bcw, Dir, cart ) :-
	!,
	atom_concat( Dir, 'carts/bcw/grow/data/bcw_data', File ),
	ensure_loaded( File ),
	bcw_data( _Ids, _Cart ).
load_data( kyph, Dir, cart ) :-
	!,
	atom_concat( Dir, 'carts/kyphosis/data/kyph_data', File ),
	ensure_loaded( File ),
	kyph_data( _Ids, _Cart ).
load_data( pima, Dir, cart ) :-
	!,
	atom_concat( Dir, 'carts/pima/data/pima_data', File ),
	ensure_loaded( File ),
	pima_data( _Ids, _Cart ).
load_data( icml_bcw, Dir, cart ) :-
	!,
	atom_concat( Dir, 'pbl/icml05/data/bcw_train_080', File ),
	ensure_loaded( File ),
	atom_concat( Dir, 'pbl/icml05/data/idsd', Idsd),
	ensure_loaded( Idsd ),
	idsd( 9, bcw_train_080, _Ids, _Cart ).
load_data( icml_kyph, Dir, cart ) :-
	!,
	atom_concat( Dir, 'pbl/icml05/data/kyph_train_080', File ),
	ensure_loaded( File ),
	atom_concat( Dir, 'pbl/icml05/data/idsd', Idsd),
	ensure_loaded( Idsd ),
	idsd( 3, kyph_train_080, _Ids, _Cart ).
load_data( icml_pima, Dir, cart ) :-
	!,
	atom_concat( Dir, 'pbl/icml05/data/pima_train_080', File ),
	ensure_loaded( File ), % 2005/09/16: not needed any more ? idsd does the loading...?
	atom_concat( Dir, 'pbl/icml05/data/idsd', Idsd),
	ensure_loaded( Idsd ),
	idsd( 8, pima_train_080, _Ids, _Cart ).
load_data( pima_synth_500, Dir, _ ) :-
	!,
	atom_concat( Dir, 'pbl/ijcai05/data/idsd', Idsd),
	ensure_loaded( Idsd ),
	idsd( 8, sg500, _Ids, _Cart ).

load_data( SNPrv, Dir, cart_red ) :-
	atom_concat( sn_, _, SNPrv ),
	!,
	( atom_concat(SN,'_yw6',SNPrv) -> 
		atom_concat( Dir, 'carts/src/cart_llhood', Cllhood ),
		ensure_loaded( Cllhood ),
		Weights = [y-6],
		set_class_weights( Weights )
		; 
		( atom_concat(SN,'_yw10',SNPrv) -> 
			atom_concat( Dir, 'carts/src/cart_llhood', Cllhood ),
			ensure_loaded( Cllhood ),
			Weights = [y-10],
			set_class_weights( Weights )
			;
			SN = SNPrv,
			Weights = []
		)
	),
	write( user_error, using_weights(Weights) ), nl,
	atom_concat( Dir, 'carts/sn/data/idsd', Idsd ),
	atom_concat( Dir, 'carts/sn/data/', DirData ),
	atom_concat( DirData, SN, SNBase ),
	elm( 'auxil/dset_has_nofeatures' ),
	dset_has_nofeatures( SNBase, NofFts ),
	ensure_loaded( Idsd ),
	/*
	SNPrs = [sn_clean2_trn_080-19,sn_clean3_trn_080-33,
	         sn_clean4_trn_080-8, sn_clean5_trn_080-15],
	( memberchk( SN-NofFts, SNPrs ) -> 
		true
		;
		throw( unknown_sn_dset(SN) )
	),		
	*/
	idsd( NofFts, SN, _Ids, _Cart ).

load_data( sn_clean1, Dir, cart_red ) :-
	!,
	atom_concat( Dir, 'carts/sn/data/idsd', Idsd ),
	ensure_loaded( Idsd ),
	idsd( 18, clean1, _Ids, _Cart ).

load_data( sn_clean1_trn, Dir, cart_red ) :-
	!,
	atom_concat( Dir, 'carts/sn/data/idsd', Idsd ),
	ensure_loaded( Idsd ),
	idsd( 18, sn_clean1_trn_080, _Ids, _Cart ).

load_data( Data, _Dir, _Type ) :-
	write( dont_know_about_data(Data) ), nl,
	fail.
