:- ensure_loaded( '../src/lib/gnucomfile' ).
:- ensure_loaded( '../src/lib/flatoms' ).

plot_metric( Opts ) :-
	( memberchk(metric(Metric),Opts) -> true ; Metric = model ),
	( memberchk(tname(Tname),Opts) -> true ; Tname = c ),
	( memberchk(steps(Steps),Opts) -> true ; Steps = 20 ),
	( memberchk(data(DataF),Opts) -> true ; 
		write( user_error, 'Option data/1 with File was expected.' ),
		nl( user_error ), abort ),
	( (\+ memberchk(low(_),Opts); \+ memberchk(high(_),Opts)) ->
		% if min_mem flag is true then do not store measure
		read_matching_terms( DataF, Tname, Metric, measure, [], Measure ),
		( memberchk(low(Low),Opts) -> true ; min_list(Measure,Low) ),
		( memberchk(high(High),Opts) -> true ; max_list(Measure,High) ),
		Step is (High - Low) / Steps,
		First is Low + Step,
		empty_buckets( First, Step, High, Empty ),
		insert_samples( Measure, Empty, Bucketed )
		;
		memberchk( low(Low), Opts ),
		memberchk( high(High), Opts ),
		empty_buckets( First, Step, High, Empty ),
		read_matching_terms( DataF, Tname, Metric, buckets, Empty, Bucketed )
	),
	mktemp( plot_metric_XXXXXX, BuckFile ),
	tell( BuckFile ),
	write_kvsb( Bucketed ),
	told,
	mktemp( gnuplot_coms_XXXXXX, GnuComsFile ),
	gnucomfile( GnuComsFile, [with(boxes)], BuckFile ),
	flatoms( ['gnuplot -persist \"',GnuComsFile,'\"'], Gplot ),
	system( Gplot ),
	sleep( 2 ),
	delete_file( GnuComsFile ),
	delete_file( BuckFile ).

read_matching_terms( DataF, Tname, Metric, Rid, E, Out ) :-
	open( DataF, read, In ),
	read( In, First ),
	initial_on_rid( Rid, First, E, Empty ),
	read_matching_terms_1( First, In, Tname, Metric, Rid, Empty, Out ),
	close( In ).

read_matching_terms_1( end_of_file, _In, _Tname, _Metric, Rid, In, Out ) :-
	!,
	tidy_up_on_rid( Rid, In, Out ).
read_matching_terms_1( Term, Stream, Tname, Metric, Rid, In, Out ) :-
	( Term =.. [Tname|Targs] ->
		Goal =.. [Metric,Targs,Curr],
		call( Goal ),
		update_on_rid( Rid, In, Curr, Nx )
		;
		Nx = In 
	),
	read( Stream, NxTerm ),
	read_matching_terms_1( NxTerm, Stream, Tname, Metric, Rid, Nx, Out ).

empty_buckets( Low, _Step, High, Empty ) :-
        Low > High,
        !,
        Empty = [].
empty_buckets( Low, Step, High, [Low-0|T] ) :-
        NxLow is Low + Step,
        empty_buckets( NxLow, Step, High, T ).

insert_samples( [], Buck, Buck ).
insert_samples( [H|T], Acc, Buck ) :-
        kvsb_add_count( Acc, H, 1, NxAcc ),
        insert_samples( T, NxAcc, Buck ).

model( [Mod], Mod ).

initial_on_rid( range, First, _, [First,First] ).
initial_on_rid( measure, _First, _, [] ).
initial_on_rid( buckets, _First, Empty, Empty ).

update_on_rid( range, [Amin,Amax], Curr, [Nmin,Nmax] ) :-
	( Amin @> Curr -> Nmin = Curr, Nmax = Amax
	                ; Nmin = Amin, 
				   ( Amax @< Curr -> Nmax = Curr ; Nmax = Amax ) ).
update_on_rid( measure, Acc, Curr, [Curr|Acc] ).
update_on_rid( buckets, Acc, Curr, Next ) :-
	kvsb_add_count( Acc, Curr, 1, Next ).

tidy_up_on_rid( range, Out, Out ).
tidy_up_on_rid( measure, Acc, Out ) :- reverse( Acc, Out ).
tidy_up_on_rid( buckets, Out, Out ).

% kvsb = kv_sorted_buckets.
kvsb_add_count( [], _Elem, _Add, [] ).
kvsb_add_count( [K-V|T], Elem, Add, Out ) :-
        K @< Elem,
        !,
        ( T == [] -> 
                V1 is V + Add,
                Out = [K-V1|TOut]
                ;
                Out = [K-V|TOut]
        ),
        kvsb_add_count( T, Elem, Add, TOut ).
kvsb_add_count( [K-V|T], _Elem, Add, Out ) :-
        V1 is V + Add,
        Out = [K-V1|T].

write_kvsb( [] ).
write_kvsb( [K-V|T] ) :-
	write( K ), write( ' ' ), write( V ), nl,
	write_kvsb( T ).

