
:- ensure_loaded( library(system) ). 			% file_exists/1.
:- ensure_loaded( '../src/init_lib' ).

:- requires( opts_have_help/1 ).
:- requires( gzip_files/1 ).
:- requires( sys_info/1 ).
:- requires( matching_terms_in_list/5 ).
:- requires( werr/2 ).

:- prolog_load_context( directory, Dir ), bb_put( mcmcms_aux_dir, Dir ).

post_create( Opts ) :-
	opts_have_help( Opts ),
	!, nl,
	write( 'post_create:    post process MCMC model files to create statistics files.' ), nl,
	write( '                It follows the mcmcms convention, where processing code is in ' ), nl, 
	write( '                create_* files. These define exec_extension/2 with *  ' ), nl,
	write( '                as their first argument.'), nl, nl,
	write( '  Options.' ), nl,
	write( '        ext/1      extension to apply. Must be present' ), nl,
	write( '        gz/1       Boolean, whether to gzip at end. Def: false' ),
     write( '      call/1       call these calls before starting. ' ),
     nl, nl, nl.


post_create( Opts ) :-
     nl,
	matching_terms_in_list( ext(Ext), Ext, Opts, Exts, NonEOpts ),
	( select(gz(GzBool),NonEOpts,NonGOpts) -> 
		true ; NonGOpts = NonEOpts ),
     call_option_calls( NonGOpts, NonCOpts ),
	bb_get( mcmcms_aux_dir, Aux  ),
	atom_concat( Aux, '/create_', AuxSl ),
	load_all_create_files( Exts, AuxSl, _LdExts ),
	call_exec_ext_on_all_files( NonCOpts, Files ),
	( GzBool==true -> gzip_files(Files) ; true ).

load_all_create_files( [], _Aux, [] ).
load_all_create_files( [H|T], Aux, Ld ) :-
	atom_concat( Aux, H, Base ),
	atom_concat( Base, '.pl', File ),
	( file_exists(File) ->
		werr( ensure_loaded( File ), info ),
		ensure_loaded( File ),
		Ld = [H|TLd]
		;
		write( user_error, 'file doesnot exist: ' ), 
		write( user_error, File ), nl( user_error ),
		write( user_error, 'skipping extension: ' ), 
		write( user_error, H ), nl( user_error ),
		Ld = TLd
	),
	load_all_create_files( T, Aux, TLd ).

call_exec_ext_on_all_files( [], [] ).
call_exec_ext_on_all_files( [H|T], ExFs ) :-
	( file_exists(H) ->
		ExFs = [H|TExFs],
		( atom_concat(Stem,'.gz',H) ->
			atom_concat( 'gunzip ', H, Gunzip ),
			sys_info( Gunzip )
			;
			Stem = H
		),
		findall( Ex, exec_extension(Ex,Stem), Exts ),
          werr( did_extensions(H,Exts), info )
		;
		( (atom_concat(Stem,'.gz',H),file_exists(Stem)) ->
		     findall( Ex, exec_extension(Ex,Stem), Exts ),
               werr( did_extensions(H,Exts), info )
               ;    % dont do anything 
               ExFs = TExFs
          )
	),
	call_exec_ext_on_all_files( T, TExFs ).

call_option_calls( [], [] ).
call_option_calls( [call(Call)|T], NonCOpts ) :-
     !,
     write( user_error, calling(Call) ), nl( user_error ),
     call(Call), 
     call_option_calls( T, NonCOpts ).
call_option_calls( [H|T], [H|R] ) :-
     call_option_calls( T, R ).
