:- module( npbc, [
		either_strictly_positive/2,
		add_hh_if/9,
		add_link_if/6,
		add_arc_if/7,
		dmess/1
		] ).

:- ensure_loaded( library(ugraphs) ).		% del_edges/3, reachable/3, add_edges/3, top_sort/2.
:- ensure_loaded( library(lists) ).		% memberchk/2, select/3.
:- ensure_loaded( library(ordsets) ).		% ord_add_element/3, ord_del_element/3.

/*
if you edit this file (even just changing its saved-at time stamp)
do the following in SICStus 
compile( rpdag_npbc ), save_modules( [npbc], rpdag_npbc ), halt.
*/

either_strictly_positive( A, _B ) :-
	A > 0, 
	!.
either_strictly_positive( _A, B ) :-
	B > 0.
% A.hh(x,y) starts

add_hh_if( Px, Nx, X, Ny, Y, Z, Xninf, InDag, OutDag ) :-
	Ny >= 2,
	either_strictly_positive( Px, Nx ),
	!,
	dmess( committed_hh(ny(Ny),px(Px),nx(Nx)) ),
	InDag = rpdag(_Pairs,InDir,_InUnd),
	reachable( Y, InDir, YReaches ),
	\+ memberchk( X, YReaches ),
	\+ memberchk( Z, YReaches ),
	add_hh_post_if( Ny, X, Y, Z, Xninf, InDag, OutDag ).
add_hh_if( _Px, _Nx, X, Ny, Y, Z, Xninf, InDag, OutDag ) :-
	add_hh_post_if( Ny, X, Y, Z, Xninf, InDag, OutDag ).

add_hh_post_if( InNy, X, Y, Z, _TheXninf, InDag, OutDag ) :-
	InDag = rpdag(Pairs,InDir,InUnd),
	once( select(Y-Yninf,Pairs,NoYPairs) ),
	once( select(X-Xninf,NoYPairs,NoXPairs) ),
	once( select(Z-Zninf,NoXPairs,NoZPairs) ),
	% remove_undirected( InUnd, Y-Z, MdUnd ),
	del_edges( InUnd, [Y-Z,Z-Y], MdUnd ),
	add_edges( InDir, [X-Y,Z-Y], MdDir ),
	Yninf = (InPaY/InPy,ChY/Cy,InNeY/InNy),
	Xninf = (PaX/Px,InChX/InCx,NeX/Nx),
	Zninf = (PaZ/Pz,InChZ/InCz,InNeZ/InNz),
	ord_add_element( InChX, Y, ChX ), Cx is InCx + 1,
	ord_add_element( InChZ, Y, ChZ ), Cz is InCz + 1,
	ord_del_element( InNeY, Z, NeY ), Ny is InNy - 1,
	ord_add_element( InPaY, X, MdPaY ),
	ord_add_element( MdPaY, Z, PaY ), Py is InPy + 2,
	ord_del_element( InNeZ, Y, NeZ ), Nz is InNz - 1,
	UpYninf = (PaY/Py,ChY/Cy,NeY/Ny),
	UpXninf = (PaX/Px,ChX/Cx,NeX/Nx),
	UpZninf = (PaZ/Pz,ChZ/Cz,NeZ/Nz),
	ord_add_element( NoZPairs, Y-UpYninf, UpYPairs ),
	ord_add_element( UpYPairs, X-UpXninf, UpXPairs ),
	ord_add_element( UpXPairs, Z-UpZninf, UpZPairs ),
	( InNy >= 2 ->
		complete( [Y], UpZPairs, MdDir, MdUnd, CompYPairs, CompYDir, CompYUnd )
		;
		CompYPairs = UpZPairs, CompYDir = MdDir, CompYUnd = MdUnd
	),
	top_sort( CompYDir, _Sorted ),
	OutDag = rpdag(CompYPairs,CompYDir,CompYUnd).
% add_hh_post_if( _Ny, X, Y, Z, Xninf, InDag, InDag ) :-

% A.hh(x,y) ends

% A.arc(x,y) starts
add_arc_if( Px, Ny, Cy, X, Y, InDag, OutDag ) :-
	Px > 0,
	( Ny > 0 ; Cy > 0),
	!,
	dmess( committed_arc(px(Px),ny(Ny)) ),
	InDag = rpdag(InPairs,InDir,InUnd),
	reachable( X, InDir, Xreaches ),
	\+ memberchk( Y, Xreaches ),
	in_pairs_add_arc( InPairs, X, Y, XYPairs ),
	add_edges( InDir, [X-Y], XYDir ),
	( Ny > 0 ->
		complete( [Y], XYPairs, XYDir, InUnd, CompYPairs, CompYDir, CompYUnd )
		;
		CompYPairs = XYPairs, CompYDir = XYDir, CompYUnd = InUnd
	),
	top_sort( CompYDir, _Sorted ),
	% write( top_sort( CompYDir, Sorted ) ), nl,
	OutDag = rpdag(CompYPairs,CompYDir,CompYUnd).
	% needed ?
	% also maybe we could do better, since cycle(s)
	% must contain X->Y or newly converted link to arc. (conplete/n)
	% actually, there a will be a set of paths "generated" by complete/n
	% only need to consider cycles that may contain those,
	% from that, just check that there is no directed cycle form
	% each endpoint to X
add_arc_if( _Px, Ny, _Cy, X, Y, InDag, OutDag ) :-
	InDag = rpdag(InPairs,InDir,InUnd),
	in_pairs_add_arc( InPairs, X, Y, XYPairs ),
	add_edges( InDir, [X-Y], XYDir ),
	% ( (Px > 0, Ny > 0) -> 
	( (Ny > 0) -> 
		complete( [Y], XYPairs, XYDir, InUnd, CompYPairs, CompYDir, CompYUnd )
		;
		CompYPairs = XYPairs, CompYDir = XYDir, CompYUnd = InUnd
	),
	top_sort( CompYDir, Sorted ),
	dmess( top_sort( CompYDir, Sorted ) ),
	OutDag = rpdag(CompYPairs,CompYDir,CompYUnd).
% A.arc(x,y) ends

% A.link(x,y) starts
add_link_if( Nx, X, Ny, Y, InDag, OutDag ) :-
	Ny > 0, Nx > 0, !,
	dmess( committed_link(ny(Ny),nx(Nx)) ),
	InDag = rpdag(InPairs,InDir,InUnd),
	reachable( X, InUnd, Reaches ),
	\+ memberchk( Y, Reaches ),
	add_link( InPairs, InDir, InUnd, X, Y, OutDag ).

add_link_if( _Nx, X, _Ny, Y, InDag, OutDag ) :-
	InDag = rpdag(InPairs,InDir,InUnd),
	add_link( InPairs, InDir, InUnd, X, Y, OutDag ).

add_link( InPairs, InDir, InUnd, X, Y, OutDag ) :-
	once( select( Y-Yninf, InPairs, NoYPairs ) ),
	% ord_del_element( InPairs, Y-Yninf, NoYPairs ),
	Yninf = (PaY/Py,ChY/Cy,NeY/Ny),
	ord_add_element( NeY, X, NwNeY ),
	NwNy is Ny + 1,
	NwYninf = (PaY/Py,ChY/Cy,NwNeY/NwNy),
	ord_add_element( NoYPairs, Y-NwYninf, NwYPairs ),
	once( select( X-Xninf, NwYPairs, NoXPairs) ),
	Xninf = (PaX/Px,ChX/Cx,NeX/Nx),
	ord_add_element( NeX, Y, NwNeX ),
	NwNx is Nx + 1,
	NwXninf = (PaX/Px,ChX/Cx,NwNeX/NwNx),
	ord_add_element( NoXPairs, X-NwXninf, NwPairs ),
	add_edges( InUnd, [X-Y,Y-X], NwUnd ),
	OutDag = rpdag(NwPairs,InDir,NwUnd).
% A.link(x,y) ends

complete( [], Pairs, Dir, Und, Pairs, Dir, Und ).
% complete( InPairs, InDir, InUnd, Y, CompPairs, CompDir, CompUnd ) :-
complete( [Y|T], InPairs, InDir, InUnd, CompPairs, CompDir, CompUnd ) :-
	pick_head_undirected( [Y], InUnd, NwArcs/[], NxUnd ),
	% add_edges( InDir, NwArcs, CompDir ),
	direct_arcs( NwArcs, InPairs, InDir, NxPairs, NxDir, New ),
	append( New, T, NxToCmp),
	complete( NxToCmp, NxPairs, NxDir, NxUnd, CompPairs, CompDir, CompUnd ).


pick_head_undirected( [], Und, Arcs/Arcs, Und ).
pick_head_undirected( [H|T], InUnd, Arcs/TArcs, CompUnd ) :-
	( select(H-HArcs,InUnd,MdUnd) ->
		directed_from_head_switches( HArcs, H, MdUnd, Arcs, NxUnd, MdArcs )
		;
		dmess( some_kind_of_warning ),
		NxUnd = InUnd,
		MdArcs = Arcs
	),
	pick_head_undirected( T, NxUnd, MdArcs/TArcs, CompUnd ).

directed_from_head_switches( [], _From, Und, Arcs, Und, Arcs ).
directed_from_head_switches( [H|T], From, InUnd, [From-H|TArcs], Und, FArcs ) :-
	del_edges( InUnd, [H-From], NxUnd ),
	directed_from_head_switches( T, From, NxUnd, TArcs, Und, FArcs ).

direct_arcs( [], Pairs, Dir, Pairs, Dir, [] ).
direct_arcs( [A-B|T], InPairs, InDir, OutPairs, OutDir, [B|Bs] ) :-
	add_edges( InDir, [A-B], NxDir ),
	direct_arc( A, B, InPairs, NxPairs ),
	direct_arcs( T, NxPairs, NxDir, OutPairs, OutDir, Bs ).

direct_arc( A, B, InPairs, OutPairs ) :-
	once( select(A-Aninf,InPairs,NoAPairs) ),
	once( select(B-Bninf,NoAPairs,NoBPairs) ),
	Aninf = (PaA/Pa,ChA/Ca,NeA/Na),
	Bninf = (PaB/Pb,ChB/Cb,NeB/Nb),
	% add A to parents of B
	ord_add_element( PaB, A, NwPaB ), NwPb is Pb + 1,
	% add B to children of A
	ord_add_element( ChA, B, NwChA ), NwCa is Ca + 1,
	% remove A from neighbours of B
	ord_del_element( NeB, A, NwNeB ), NwNb is Nb - 1,
	% remove B from neighbours of A
	ord_del_element( NeA, B, NwNeA ), NwNa is Na - 1,
	NwAninf = (PaA/Pa,NwChA/NwCa,NwNeA/NwNa),
	NwBninf = (NwPaB/NwPb,ChB/Cb,NwNeB/NwNb),
	% check that the following works as expected, ord_del_element/3 doesnt
	ord_add_element( NoBPairs, A-NwAninf, WtAPairs ),
	ord_add_element( WtAPairs, B-NwBninf, OutPairs ).

in_pairs_add_arc( InPairs, A, B, ABPairs ) :-
	once( select(A-Aninf,InPairs,NoAPairs) ),
	once( select(B-Bninf,NoAPairs,NoBPairs) ),
	Aninf = (PaA/Pa,ChA/Ca,NeA/Na),
	Bninf = (PaB/Pb,ChB/Cb,NeB/Nb),
	% Add A to the parents of B
	ord_add_element( PaB, A, NwPaB ), NwPb is Pb + 1,
	% Add B to the children of A
	ord_add_element( ChA, B, NwChA ), NwCa is Ca + 1,
	NwAninf = (PaA/Pa,NwChA/NwCa,NeA/Na),
	NwBninf = (NwPaB/NwPb,ChB/Cb,NeB/Nb),
	% check that the following works as expected, ord_del_element/3 doesnt
	ord_add_element( NoBPairs, A-NwAninf, WtAPairs ),
	ord_add_element( WtAPairs, B-NwBninf, ABPairs ).

dmess( _ ).
% dmess( Mess ) :-
	% write( Mess ), nl.
