
:- ensure_loaded( '../../src/init_lib' ). % requires/1.

:- requires( opts_select_infiles/3 ).
:- requires( load_1_loads/2 ).
:- requires( file_to_list_of_lines/2 ).
:- requires( parse_next_integer/3 ).
:- requires( until/5 ).
:- requires( parse_atom/3 ).
:- requires( skip_code/2 ).
:- requires( parse_number/3 ).
:- requires( memberchk/2 ).
:- requires( member/2 ).
:- requires( nth/3 ).

r_cart_to_t_cart( Opts ) :-
	opts_select_infiles( Opts, Fs, NfOpts ),
	load_1_loads( NfOpts, OthOpts ),
	r_cart_files_to_t_carts( Fs, OthOpts ).

r_cart_files_to_t_carts( [], _OthOpts ).
r_cart_files_to_t_carts( [F|Fs], OthOpts ) :-
	r_cart_to_t_cart( F, TCart ),
	( atom_concat(Stem,'.txt',F) -> true; Stem = F ),
	atom_concat( Stem, '.mvisits', CrFile ),
	open( CrFile, write, CrOut ), 
	write( 1-c(TCart) ), nl,
	write( CrOut, 1-c(TCart) ), write( CrOut, '.' ), nl( CrOut ),
	close( CrOut ),
	r_cart_files_to_t_carts( Fs, OthOpts ).

r_cart_to_t_cart( RCartFile, TCart ) :-
	file_to_list_of_lines( RCartFile, RLines ),
	r_cart_lines_to_t_cart_tokens( RLines, TLines ),
	parse_t_cart_lines( TLines, TCart ).

r_cart_lines_to_t_cart_tokens( [], [] ).
r_cart_lines_to_t_cart_tokens( [H|T], [Ht|Tt] ) :-
	r_cart_line_to_t_cart_tokens( H, Ht ),
	r_cart_lines_to_t_cart_tokens( T, Tt ).

r_cart_line_to_t_cart_tokens( H, TT ) :-
	TT = tt(Node,Var,Op,Val,Pop,Loss,Yval,Yprob,Leaf),
	parse_next_integer( Node, H, H1 ),
	% write( node(Node) ),
	H1 = [0')|H2],
	until( parse_atom, H2, nths(1,1,[H3]), skip_code, Var ), 
	% write( -var(Var) ),
	( Var == root -> 
		Op = root,
		Val= root,
		H5 = H3
		;
		H3 = [OpC1,OpC2|H3a],
		( OpC1 == 0'= -> 
			Op = =,
			H4 = [OpC2|H3a]
			;
			H4 = H3a,
			once( operator_codes_to_term( [OpC1,OpC2], Op ) )
		),
		( parse_atom(Val,H4,H5) ->
			true
			;
			parse_number( Val, H4, H5 )
		)
	),
	% write( -op(Op) ),
	% write( -val(Val) ),
	parse_next_integer( Pop, H5, H6 ),
	% write( -pop(Pop) ),
	parse_next_integer( Loss, H6, H7 ),
	% write( -loss(Loss) ),
	H7 = [0' |H8],
	( parse_atom(Yval, H8,H9) ->
		% write( atom(Yval) ), nl,
		true
		;
		parse_next_integer( Yval, H8, H9 )
		% write( integer(Yval) ), nl
	),
	% write( -yval(Yval) ),
	H9 = [0' ,0'(|H10],
	parse_r_cart_distro( Yprob, H10, H11 ),
	% write( -yprob(Yprob) ),
	( memberchk(0'*,H11) -> 
		Leaf = t
		;
		Leaf = f
	).
	% write( lf(Leaf) ), nl.

operator_codes_to_term( "< ", < ).
operator_codes_to_term( ">=", >= ).

parse_r_cart_distro( Prbs, In, Rest ) :-
	In = [0' |Rem],
	!,
	parse_r_cart_distro( Prbs, Rem, Rest ).
parse_r_cart_distro( Prbs, In, Rest ) :-
	In = [0')|Rest],
	!,
	Prbs = [].
parse_r_cart_distro( [Fst|TPrbs], In, Rest ) :-
	parse_number( Fst, In, Rem ),
	parse_r_cart_distro( TPrbs, Rem, Rest ).

%% parse_t_cart_lines( TLines, TCart ).
parse_t_cart_lines( [_Root|Rest], TCart ) :-
	findall( Vars, cart_variables_in_order(Vars), AllVarSets ),
	( AllVarSets = [VarsOrd] -> true; 
		( AllVarsSets == [] ->
			throw( no_clause_for(cart_variables_in_order/1) )
			;
			length( AllVarsSets, AVSsLgt ),
			throw( too_many_clauses_for(AVSsLgt,1,cart_variables_in_order/1) )
		)
	),
	select_t_line( Rest, 2, Line2, Rest2 ),
	select_t_line( Rest2, 3, Line3, Rest3 ),
	Line2 = tt(2,Var,_Op2,Val2,_Pop2,_Loss2,_Yval2,_Yprob2,Leaf2),
	Line3 = tt(3,Var,_Op3,Val3,_Pop3,_Loss3,_Yval3,_Yprob3,Leaf3),
	parse_t_line_branch( Leaf2, Rest3, 2, VarsOrd, Branch2, Rest4 ),
	parse_t_line_branch( Leaf3, Rest4, 3, VarsOrd, Branch3, _Rest5 ),
	( Val2 == Val3 -> 
		Val = Val2, LBranch = Branch2, RBranch = Branch3
		;
		( Val2 @< Val3 ->
			Val = Val2, LBranch = Branch2, RBranch = Branch3
			;
			Val = Val3, LBranch = Branch3, RBranch = Branch2
		)
	),
	r_variable_to_numeric_id( Var, VarsOrd, VarNumId ),
	r_value_to_t_value( VarNumId, Val, Tval ),
	TCart = cr(VarNumId,Tval,LBranch,RBranch).

parse_t_line_branch( f, TLines1, N, VarsOrd, TCart, Rem ) :-
	!,
	Ndbl is integer(N * 2),
	NdblPlus is Ndbl + 1,
	select_t_line( TLines1, Ndbl, Line2, TLines2 ),
	select_t_line( TLines2, NdblPlus, Line3, TLines4 ),
	Line2 = tt(Ndbl,Var,_Op2,Val2,_Pop2,_Loss2,_Yval2,_Yprob2,Leaf2),
	Line3 = tt(NdblPlus,Var,_Op3,Val3,_Pop3,_Loss3,_Yval3,_Yprob3,Leaf3),
	parse_t_line_branch( Leaf2, TLines4, Ndbl, VarsOrd, Branch2, TLines5 ),
	parse_t_line_branch( Leaf3, TLines5, NdblPlus, VarsOrd, Branch3, Rem ),
	( Val2 == Val3 -> 
		Val = Val2, LBranch = Branch2, RBranch = Branch3
		;
		( Val2 @< Val3 ->
			Val = Val2, LBranch = Branch2, RBranch = Branch3
			;
			Val = Val3, LBranch = Branch3, RBranch = Branch2
		)
	),
	r_variable_to_numeric_id( Var, VarsOrd, VarNumId ),
	r_value_to_t_value( VarNumId, Val, Tval ),
	TCart = cr(VarNumId,Tval,LBranch,RBranch).

parse_t_line_branch( t, TLines, _N, _VarsOrd, l, TLines ).

select_t_line( [H|T], Id, Line, RemLines ) :-
	arg( 1, H, Id ),
	!,
	Line = H,
	RemLines = T.
select_t_line( [H|T], Id, Line, [H|TRemLines] ) :-
	select_t_line( T, Id, Line, TRemLines ).

r_variable_to_numeric_id( Var, VarsOrd, VarNumId ) :-
	nth( VarNumId, VarsOrd, Var ).

r_value_to_t_value( Var, Val, Tval ) :-
	write( looking_for(Var,Val) ), nl,
	bb_get( Var, VarIndex ),
	% write( in(VarIndex) ), nl,
	VarIndex = [FstK-_|TKVs],
	kv_find_greatest_leq_k_than( TKVs, Val, FstK, Tval ).

kv_find_greatest_leq_k_than( [], KPvt, LstK, GrLow ) :-
	GrLow = LstK. 
	% when all ids in a branch are of single value.
kv_find_greatest_leq_k_than( [K-_V|T], KPvt, LstK, GrLow ) :-
	K > KPvt,
	!,
	GrLow = LstK.
kv_find_greatest_leq_k_than( [K-_V|T], KPvt,  _LstK, GrLow ) :-
	kv_find_greatest_leq_k_than( T, KPvt, K, GrLow ).
