/*

collate_within_boundary_with_extremes( 
     [Val-Id|Pairs]
     Lim                    minimum extremeties populations
     Sum                    number of pairs (1st argument)
     Lxtrm                  partial list with left extremety
     LTail                  tail of the left extremety list
     Rxtrm                  right extremety list
     LxLen                  length of left extremety
     RxLen                  length of right extremety
     Trips                  vidsl(Val,Ids,Len) triplets list
     Tot                    is the total of Len in the vidsl/3s 
     ) :-

     Create Value split (vidsl) points from Val-Id pairs. Ensures that 
     each point will create partitions of at least Lim length.
     Partitions are considered as =< Val for left and >Val for right.
     Pairs that cannot create valid partitions go to Lx (left xtremity)
     and Rx.

     */

/*
collate_within_boundary_with_extremes( +Pairs, +Lim, +Sum, -Lxtrm, -Rxtrm, -Trips, -Tot ) :-

collate_within_boundary_with_extremes( [a-2,b-1,b-3,c-5,d-6], 2, 5, L, [], R, LLen, RLen, Tri, Tot ).
collate_within_boundary_with_extremes( [a-2,b-1,b-3,c-5,d-6,e-4], 2, 6, L, [],  R, LLen, RLen, Tri, Tot ).
collate_within_boundary_with_extremes( [a-2,b-1,b-3,c-4,c-5,c-6], 2, 6, L, [], R, LLen, RLen, Tri, Tot ).
collate_within_boundary_with_extremes( [c-2,c-1,c-3,c-4,c-5,c-6], 2, 6, L, [], R, LLen, RLen, Tri, Tot ). % should fail

     */

collate_within_boundary_with_extremes( [K-V|T], Lim, Sum, Lxtrm, LTail, Rxtrm, LxLen, RxLen, Trips, Tot ) :-
     collect_first_k_vs( T, K, 1, Rem, TVs, Len ),
     collate_left_extreme( Len, Lim, 0, K, [V|TVs], Len, Rem, Lxtrm, LTail, LxLen, K1, Vs1, Len1, Rem1 ),
     InBound is Sum - LxLen - Len1 - Lim,
     % 0 =< InBound,
     collate_within_boundary( InBound, 0, K1, Vs1, Len1, Rem1, Trips, Tot, _K2, Vs2, Len2, Rem2 ),
     collate_right_extreme( Rem2, Vs2, Len2, 0, Rxtrm, RxLen ).
     
collate_within_boundary( InB, Acc, K, Vs, Len, Rem, [], Tot, K2, Vs2, Len2, Rem2 ) :-
     InB < 0,
     !,
     Tot is Acc,
     K2 = K, Vs2 = Vs, Len2 is Len,
     Rem2 = Rem.
collate_within_boundary( InB, Acc, K, Vs, Len, List, Trips, Tot, K2, Vs2, Len2, Rem2 ) :-
     List = [K1-V1|T1],
     !,
     Trips = [vidsl(K,Vs,Len)|TTrips],
     collect_first_k_vs( T1, K1, 1, Rem1, TVs1, Len1 ),
     NxB is InB - Len,   % THIS is wrong, Len1 is the correct here we are trying to reproduce an earlier faulty version.
     NxAcc is Acc + Len,
     collate_within_boundary( NxB, NxAcc, K1, [V1|TVs1], Len1, Rem1, TTrips, Tot, K2, Vs2, Len2, Rem2 ).
collate_within_boundary( _InB, Acc, K, Vs, Len, [], Trips, Tot, _, [], 0, [] ) :-
     Trips = [vidsl(K,Vs,Len)],
     Tot is Acc + Len.


collate_left_extreme( Acc, Lim, Cov, K, Vs, Len, List, Lxtrm, Ltail, LxLen, K1, Vs1, Len1, Rem1 ) :-
     Acc > Lim, 
     !,
     Lxtrm = Ltail,
     LxLen is Cov,
     K1 = K,
     Vs1 = Vs,
     Len1 is Len,
     Rem1 = List.
collate_left_extreme( Acc, Lim, _Cov, _K, Vs, _Len, List, Lxtrm, Ltail, LxLen, KOut, VsOut, LenOut, RemOut ) :-
     List = [K1-V1|T1],
     thread_list( Vs, Lxtrm, TLxtrm ),
     collect_first_k_vs( T1, K1, 1, Rem1, TVs1, Len1 ),
     NxAcc is Acc + Len1,
     collate_left_extreme( NxAcc, Lim, Acc, K1, [V1|TVs1], Len1, Rem1, TLxtrm, Ltail, LxLen, KOut, VsOut, LenOut, RemOut ).

collate_right_extreme( [], Vs, Len, Acc, Rxtrm, RxLen ) :-
     Rxtrm = Vs,
     RxLen is Len + Acc.
collate_right_extreme( [K1-V1|T1], Vs, Len, Acc, Rxtrm, RxLen ) :-
     thread_list( Vs, Rxtrm, TRxtrm ),
     NxAcc is Acc + Len,
     collect_first_k_vs( T1, K1, 1, Rem1, TVs1, Len1 ),
     collate_right_extreme( Rem1, [V1|TVs1], Len1, NxAcc, TRxtrm, RxLen ).


collect_first_k_vs( [], _K, Len, [], [], Len ).
collect_first_k_vs( [K-V1|T], K, Acc, Rem, [V1|TVs], Len ) :-
     !,
     Nxt is Acc + 1,
     collect_first_k_vs( T, K, Nxt, Rem, TVs, Len ).
collect_first_k_vs( Rem, _K, Len, Rem, [], Len ).

thread_list( [], Tail, Tail ).
thread_list( [H|T], [H|R], Tail ) :-
     thread_list( T, R, Tail ).
