:- ensure_loaded( library(lists) ).			% memberchk/2,
 										% working_directory/2.
 										% append/3.
:- ensure_loaded( '../../src/init_lib' ).		%.
:- requires( directory_files_regular/2 ).
:- requires( break_nth/4 ).
:- requires( contains/2 ).
:- requires( file_stems_of_files_with_this_ext/3 ).

jmlr_table :-
	findall( Prior, prior_dir(Prior), Priors ),
	priors_exp_matrix( Priors, PriorsMtx ),
	TrueBNs = [as,al,hf,sc,p1,p2],
	open( 'LossesTable.txt', write, Out ),
	write( Out, '            id            ed            ne                  ks                ck' ), nl( Out ),
	write_priors_matrix( TrueBNs, PriorsMtx, Out ), 
	close( Out ).

write_priors_matrix( [], _Mtx, _Out ).
write_priors_matrix( [HBn|TBns], PrMtx, Out ) :-
	all_bn_entries( PrMtx, HBn, Bn500Ents, Bn1KEnts ),
	true_bn_name( HBn, BnName ),
	write( Out, BnName ), nl( Out ),
	write( Out, 'datapoints: 500' ), nl( Out ),
	% write( Out, 'Prior' ),
	write_bn_entries( Bn500Ents, Out ),
	nl( Out ),
	write( Out, 'datapoints: 1000' ), nl( Out ),
	% write_loss_spaces( 9, Out ),
	% write( Out, 'Prior' ),
	write_bn_entries( Bn1KEnts, Out ),
	nl( Out ),
	write_priors_matrix( TBns, PrMtx, Out ).

all_bn_entries( [], _HBn, [], [] ).
all_bn_entries( [HPr/HPrEnt|T], HBn, [HPr/H500|TBn500Ents], [HPr/H1K|TBn1KEnts] ) :-
	findall( row(Lid,A,B,C)-row(Lid,D,E,F), 
				member(HBn+row(Lid,A,B,C,D,E,F),HPrEnt),
						RowPrs ),
	split_bn_entries( RowPrs, H500, H1K ),
	% H500 = HPr/row(Lid,A,B,C),
	% H1K  = HPr/row(Lid,D,E,F),
	all_bn_entries( T, HBn, TBn500Ents, TBn1KEnts ).

split_bn_entries( [], [], [] ).
split_bn_entries( [H5-H1|T], [H5|T5], [H1|T1] ) :-
	split_bn_entries( T, T5, T1 ).

write_bn_entries( [], _Out ).
write_bn_entries( [HPr/Rows|T], Out ) :-
% write_bn_entries( [HPr/row(A,B,C)|T], Out ) :-
	atom_concat( prior, PriorId, HPr ),
	write( Out, '  ' ), write( Out, PriorId ), write( Out, ' : ' ),
	% write_bn_losses( A, B, C ), 
	Losses = [id,ed,ne,ks,ck],
	losses_in_rows( Losses, Rows, Out ),
	nl( Out ),
	% loss_in_row( id, A, B, C, IdLoss ),
	% loss_in_row( ed, A, B, C, EdLoss ),
	% loss_in_row( ne, A, B, C, NeLoss ),
	% loss_in_row( ks, A, B, C, KsLoss ),
	% loss_in_row( ck, A, B, C, CkLoss ),
	% write_loss_numbers( IdLoss, 3, Out ),
	% write_loss_numbers( EdLoss, 3, Out ),
	% write_loss_numbers( NeLoss, 5, Out ),
	% write_loss_numbers( KsLoss, 5, Out ),
	% write_loss_numbers( CkLoss, 5, Out ),
	% nl( Out ),
	write_bn_entries( T, Out ).

losses_in_rows( [], _Rows, _Out ).
losses_in_rows( [H|T], Rows, Out ) :-
	% H500 = HPr/row(Lid,A,B,C),
	% loss_in_row( id, A, B, C, IdLoss ),
	memberchk( row(H,A,B,C), Rows ),
	( (H==id;H==ed) -> Lgt = 3; Lgt = 5 ),
	write_loss_numbers( [A,B,C], Lgt, Out ),
	losses_in_rows( T, Rows, Out ).


%here
loss_in_row( Name, A, B, C, _Loss ) :-
	loss_memberchk( Name, A, AArg ),
	loss_memberchk( Name, B, BArg ),
	loss_memberchk( Name, C, CArg ),
	_LoABC = [AArg,BArg,CArg].

loss_memberchk( Name, List, Arg ) :-
	( List == na -> 
		Arg = na 
		;
		Item =.. [Name,Arg], 
		( memberchk(Item,List) ->
			true
			;
			Arg = na
		)
	).

%here
write_loss_numbers( [], _Lgth, _Out ).
write_loss_numbers( [HPrv|T], Lgth, Out ) :-
	( HPrv == na ->
		RedLgth is Lgth - 1,
		write_loss_spaces(RedLgth,Out),
		write( Out, '_' )
		;
		( HPrv =:= integer(HPrv) -> H is integer(HPrv); H=HPrv ),
		number_codes( H, HCsPrv ),
		( HCsPrv = [0'0|HCsPrv2] -> 
			( HCsPrv2 == [] ->
				HCs = HCsPrv 
				;
				HCs = HCsPrv2
			)
			;
			HCs = HCsPrv
		),
		length( HCs, HLgth ),
		( HLgth > Lgth ->
			break_nth( Lgth, HCs, HoutCs, _ )
			;
			HCs = HoutCs,
			( HLgth < Lgth -> 
				Diff is Lgth - HLgth,
				write_loss_spaces(Diff,Out)
				;
				true
			)
		),
		atom_codes( Hout, HoutCs ),
		write( Out, Hout )
	),
	( T == [] -> write(Out,' : ') ; write(Out,',') ),
	write_loss_numbers( T, Lgth, Out ).

priors_exp_matrix( [], [] ).
priors_exp_matrix( [H|T], [HRow|TRows] ) :-
	atom_concat( H, '/losses', LossesDir ),
	working_directory( Top, LossesDir ),
	TrueBNs = [as,al,hf,sc,p1,p2],
	directory_files_regular( '.', All ),
	% file_stems_of_files_with_this_ext( All, '_bst_mst.pl', BstMst ),
	atom_codes_list( All, AllCs ),
	Losses = [id,ed,ne,ks,ck],
	prior_exp_multi_row( Losses, TrueBNs, AllCs, [], Row ),
	working_directory( _, Top ),
	HRow = H/Row,
	priors_exp_matrix( T, TRows ).

prior_exp_multi_row( [], _TrueBNs, _Files, Row, Row ).
prior_exp_multi_row( [H|T], TrueBNs, Files, Acc, Row ) :-
	atom_concat( '.e', H, LossExt ),
	atom_codes( LossExt, LossExtCs ),
	prior_exp_row( TrueBNs, LossExtCs/H, Files, F ),
	append( Acc, F, NxAcc ),
	prior_exp_multi_row( T, TrueBNs, Files, NxAcc, Row ).

prior_exp_row( [], _Loss/_H, _FilesCs, [] ).
prior_exp_row( [HBn|TBns], L/Lid, Files, [HBn+HEnt|TRows] ) :-
	atom_codes( HBn, HBnCsPrv ),
	append( "cm_t", HBnCsPrv, HBnCs ),
	losses_mtx_entry( Files    , HBnCs, "d500", "s103", L, A, RemFilesA ),
	losses_mtx_entry( RemFilesA, HBnCs, "d500", "s451", L, B, RemFilesB ),
	losses_mtx_entry( RemFilesB, HBnCs, "d500", "s772", L, C, RemFilesC ),
	losses_mtx_entry( RemFilesC, HBnCs, "d1K", "s103", L, D, RemFilesD ),
	losses_mtx_entry( RemFilesD, HBnCs, "d1K", "s451", L, E, RemFilesE ),
	losses_mtx_entry( RemFilesE, HBnCs, "d1K", "s772", L, F, RemFilesF ),
	HEnt = row(Lid,A,B,C,D,E,F),
	prior_exp_row( TBns, L/Lid, RemFilesF, TRows ).

losses_mtx_entry( [], _TrBn, _Data, _Seed, _Loss, na, [] ).
losses_mtx_entry( [H|T], TrBn, Data, Seed, Loss, Ent, Rem ) :-
	contains( H, TrBn ), 
	contains( H, Data ), 
	contains( H, Seed ),
	contains( H, Loss ),
	!,
	atom_codes( HAtm, H ),
	% atom_concat( HAtm, '_bst_mst.pl', File ),
	open( HAtm, read, In ),
	read( In, expected(Ent,_) ),
	% read( In, mst_visited_losses(MstLosses,_,_,_) ),
	% Ent = BstLosses,
	close( In ),
	Rem = T.
losses_mtx_entry( [H|T], TrBn, Data, Seed, Loss, Ent, [H|Rem] ) :-
	losses_mtx_entry( T, TrBn, Data, Seed, Loss, Ent, Rem ).

prior_dir( prior01 ).
prior_dir( prior02 ).
prior_dir( prior03 ).
prior_dir( prior04 ).
prior_dir( prior05 ).
prior_dir( prior06 ).
prior_dir( prior07 ).
prior_dir( prior08 ).
prior_dir( prior09 ).

true_bn_name( as, 'Asia      ' ).
true_bn_name( al, 'Alarm     ' ).
true_bn_name( hf, 'Hailfinder' ).
true_bn_name( sc, 'Insurance ' ).
true_bn_name( p1, 'Parity1   ' ).
true_bn_name( p2, 'Parity2   ' ).

atom_codes_list( [], [] ).
atom_codes_list( [H|T], [HCs|TCs] ) :-
	atom_codes( H, HCs ),
	atom_codes_list( T, TCs ).

write_loss_spaces( 0, _Out ) :- !.
write_loss_spaces( N, Out ) :-
	write( Out, ' ' ),
	NxN is N - 1,
	write_loss_spaces( NxN, Out ).
