:- ensure_loaded( library(lists) ).		% memberchk/2, append/3.
:- ensure_loaded( '../../src/init_lib' ).
:- ensure_loaded( library(read_terms) ).			% /2.
:- ensure_loaded( library(select_compound_w_gunzip) ).	% /4.
:- ensure_loaded( library(gzip_files) ).			% /1.

loss_files_to_excl_n_exp_loss_points( Opts ) :-
	( Opts==h; Opts==help; memberchk(h,Opts); memberchk(help,Opts) ),
	!, nl,
	write( 'loss_files_to_excl_n_exp_loss_points:    a script for producing excluding-n, expected' ), nl, 
	write( '                                  loss step files from loss files.' ), nl,

	write( '                                  Its output files can be fed into'), nl,
	write( '                                  mcmcms/auxil/gsplay.pl .' ), nl,
	write( ' Options.' ), nl,
	write( '       compress/1     whether to use gzip on the output. Def. is true.' ), nl,
	write( '                      and at least one should be given.' ), nl,
	write( '       out/1          returns output filenames.' ), nl,
	write( '       step/1         exclusion step. Default is 1.' ), nl,
		write( '                      Non compound options are taken to be input loss files.' ), nl, nl.

loss_files_to_excl_n_exp_loss_points( Opts ) :-
	( memberchk(step(Step),Opts) -> 
		true
		;
		write( using_default_step(1) ), nl,
		Step = 1
	),
	% findall( F, member(lfile(F),Opts), NestFs ),
	select_compound_w_gunzip( Opts, _, Fs, Gunzipped ),
	% flatten( NestFs, Fs ),
	% ( Fs = [] ->
		% write( user_error, 'Didnt find any lfile/1 options, quiting.'),
		% nl( user_error )
		% ;
		( memberchk(compress(Bgz),Opts) -> true ; Bgz = true ),
		loss_files_to_excl_n_exp_loss_point_files( Fs, Step, Bgz, OutFs ),
	% ),
	gzip_files( Gunzipped ),
	( memberchk(out(OutFs),Opts) -> true ; true ).

loss_files_to_excl_n_exp_loss_point_files( [], _Step, _Bgz, [] ).
loss_files_to_excl_n_exp_loss_point_files( [F|Fs], Step, Bgz, [OutH|OutT] ) :-
	( atom_concat( Stem, '.gz', F ) -> 
		atom_concat( 'gunzip ', F, Gunzip ),
		system( Gunzip ),
		loss_file_to_exp_loss_point_file( Stem, Step, PntsFile ),
		atom_concat( 'gzip ', Stem, Gzip ),
		system( Gzip )
		;
		loss_file_to_excl_n_exp_loss_point_file( F, Step, PntsFile )
	),
	( Bgz == true -> 
		atom_concat( 'gzip ', PntsFile, GzipPoints ), 
		system( GzipPoints ),
		atom_concat( PntsFile, '.gz', OutH )
		;
		OutH = PntsFile
	),
	loss_files_to_excl_n_exp_loss_point_files( Fs, Step, Bgz, OutT ).

loss_file_to_excl_n_exp_loss_point_file( File, Step, ExtFile ) :-
	number_codes( Step, StepCs ),
	% atom_codes( Ext, [0'_,0'x|StepCs] ),
	atom_codes( File, FileCs ),
	append( Stem, [0'.,0'l,A,B], FileCs ),
	append( Stem, [0'.,0'g,A,B,0'_,0'x|StepCs], ExtFileCs ),
	% atom_concat( File, Ext, FileExt ),
	atom_codes( ExtFile, ExtFileCs ),
	open( File, read, In ),
	open( ExtFile, write, Out ),
	read_term( In, T, [] ),
	loss_fname_to_exp_sum_and_length( File, Tot, Lgt ),
	EPoint is Tot / Lgt,
	write( Out, EPoint ), nl( Out ),
	loss_stream_to_excl_n_exp_loss_point_stream( T, 1, Step, Tot, Lgt, 0, 0, In, Out ),
	close( In ),
	close( Out ).

loss_stream_to_excl_n_exp_loss_point_stream( end_of_file, I, _S, T, G, Sum, It, _In, Out ) :-
	!,
	( I =:= 1 ->
		true
		;
		EPoint is Sum / It,
		write( Out, EPoint ), nl( Out )
	),
	( G =:= It -> true ; 
		write( user_error, length_of_chain_clash(G,It) ), nl( user_error ) ),
	( T =:= Sum -> true ; 
		write( user_error, sum_of_chain_clash(T,Sum) ), nl( user_error ) ).
loss_stream_to_excl_n_exp_loss_point_stream( l(L), I, I, T, G, Sum, It, In, Out ) :-
	!,
	NxSum is Sum + L,
	NxIt is It + 1,
	EPoint is (T - Sum) / (G - It),
	write( Out, EPoint ), nl( Out ),
	read_term( In, RdT, [] ),
	loss_stream_to_excl_n_exp_loss_point_stream( RdT, 1, I, T, G, NxSum, NxIt, In, Out ).
loss_stream_to_excl_n_exp_loss_point_stream( l(L), I, St, T, G, Sum, It, In, Out ) :-
	NxSum is Sum + L, 
	NxIt is It + 1,
	NxI is I + 1,
	read_term( In, RdT, [] ),
	loss_stream_to_excl_n_exp_loss_point_stream( RdT, NxI, St, T, G, NxSum, NxIt, In, Out ).

loss_fname_to_exp_sum_and_length( File, Sum, Lgt ) :-
	atom_codes( File, FileCs ),
	append( Base, [0'.,0'l,X,Y], FileCs ),
	append( Base, [0'.,0'e,X,Y], ExpFileCs ),
	atom_codes( ExpFile, ExpFileCs ),
	read_terms( ExpFile, ExpTerms ),
	memberchk( expected(_Val,Sum/Lgt), ExpTerms ).
