:- ensure_loaded( library(lists) ).         % append/3.

:- requires( file_to_list_of_lines/2 ).
:- requires( break_list_on/4 ).
:- requires( write_list_elem_per_line/1 ).
:- requires( flatoms/2 ).
:- requires( kv_transpose/2 ).
:- requires( kv_decompose/3 ).

runtimes( [File] ) :-
     file_to_list_of_lines( File, Lines ),
     tail_lines_to_pl( Lines, PlTerms ),
     keysort( PlTerms, Sorted ),
     % write_list_elem_per_line( ByPrior ).
     kv_average_v_for_k( Sorted, Averaged ),
     % write_list_elem_per_line( Averaged ),
     select_size_true( Averaged, 500, alarm, Alarm500 ),
     flatoms( ['500_alarm.dat'], Alarm500File ),
     record_on_file( Alarm500File, Alarm500, 1 ),
     select_size_true( Averaged, 1000, alarm, Alarm1k ),
     flatoms( ['1000_alarm.dat'], Alarm1kFile ),
     record_on_file( Alarm1kFile, Alarm1k, 1 ),
     select_size( Averaged, 1000, OneK ),
     select_prior_times_on_true_reduced( OneK, alarm, OneKAlarm ),
     write( onekalarm(OneKAlarm) ), nl,
     keysort( OneKAlarm, SortedOKA ),
     kv_decompose( SortedOKA, _AlarmTms, RevProtoPriors ),
     reverse( RevProtoPriors, ProtoPriors ),
     record_recombined( OneK, ProtoPriors ).

record_recombined( [], _ProtoPriors ).
record_recombined( [H|T], ProtoPriors ) :-
     % OneK = [(Prior,True)-Time|TOneK],
     H = (Prior,True)-Time,
     select_reduced_on_true( T, True, These, Rem ),
     write( proto(ProtoPriors) ), nl,
     sort_kv_on_proto_ks( ProtoPriors, [Prior-Time|These], Sorted ),
     atom_concat( True, '_recombined.dat', File ),
     record_on_file( File, Sorted, 1 ),
     % open( File, write, Out ),
     record_recombined( Rem, ProtoPriors ).

sort_kv_on_proto_ks( [], [], [] ).
sort_kv_on_proto_ks( [H|T], List, [Htime|Ttimes] ) :-
     select( H-Htime, List, Rem ),
     !,
     sort_kv_on_proto_ks( T, Rem, Ttimes ).


select_reduced_on_true( [], _True, [], [] ).
select_reduced_on_true( [(Prior,ThisTrue)-Time|T], True, These, Rem ) :-
     ( ThisTrue == True -> 
          These = [Prior-Time|TThese],
          Rem   = TRem
          ;
          These = TThese,
          Rem   = [(Prior,ThisTrue)-Time|TRem]
     ),
     select_reduced_on_true( T, True, TThese, TRem ).

tail_lines_to_pl( [], [] ).
tail_lines_to_pl( [[0'=,0'=,0'>,0' |RstLn]|T], PlTerms ) :-
     !,
     break_list_on( RstLn, 0'/, PriorCs, ModelsDirStatsFile ),
     atom_codes( Prior, PriorCs ),
     % write( prior(Prior) ), nl,
     break_list_on( ModelsDirStatsFile, 0'/, _ModelsDir, StatsFile ),
     append( "cm_t", [Fst,Sec|DAnd], StatsFile ),
     append( "_d",   [DFs,DSe|_],    DAnd      ),
     once( true_abbrv_to_full([Fst,Sec], True) ),
     once( dsize_abbrv_to_full([DFs,DSe], DSize) ),
     T = [TimeLn|Rem],
     number_codes( Time, TimeLn ),
     PlTerms = [(DSize,Prior,True)-Time|TPlTerms],
     % PlTerms = [Prior-tt(True,DSize,Time)|TPlTerms],
     tail_lines_to_pl( Rem, TPlTerms ).
tail_lines_to_pl( [_|T], PlTerms ) :-
     tail_lines_to_pl( T, PlTerms ).

record_on_file( File, List, N ) :-
     open( File, write, Out ),
     record_on_stream( List, N, Out ),
     close( Out ).

record_on_stream( [], _N, _Out ).
record_on_stream( [H|T], N, Out ) :-
     ( number(N) -> 
          NxN is N + 1,
          write( Out, N ), write( Out, '  ' )
          ;
          true
     ),
     write( Out, H ), nl( Out ),
     record_on_stream( T, NxN, Out ).

select_size_true( [], _Size, _True, [] ).
select_size_true( [(Size,_,True)-Time|T], Size, True, [Time|Rst] ) :-
     !,
     select_size_true( T, Size, True, Rst ).
select_size_true( [_H|T], Size, True, Rst ) :-
     select_size_true( T, Size, True, Rst ).

select_size( [], _Size, [] ).
select_size( [(Size,Prior,True)-Time|T], Size, OneK ) :-
     !,
     OneK = [(Prior,True)-Time|TOneK],
     select_size( T, Size, TOneK ).
select_size( [_|T], Size, OneK ) :-
     select_size( T, Size, OneK ).

select_prior_times_on_true_reduced( [], _True, [] ).
select_prior_times_on_true_reduced( [(Prior,True)-Time|T], True, As ) :-
     !,
     As = [Time-Prior|TAs],
     select_prior_times_on_true_reduced( T, True, TAs ).
select_prior_times_on_true_reduced( [_H|T], True,  As ) :-
     select_prior_times_on_true_reduced( T, True, As ).

true_abbrv_to_full( "al", alarm         ).
true_abbrv_to_full( "as", asia          ).
true_abbrv_to_full( "hf", hailfinder    ).
true_abbrv_to_full( "sc", insurance     ).
true_abbrv_to_full( "p1", parity1       ).
true_abbrv_to_full( "p2", parity2       ).

dsize_abbrv_to_full( "50", 500 ).
dsize_abbrv_to_full( "1K", 1000 ).

kv_average_v_for_k( [], [] ).
kv_average_v_for_k( [K-V|T], Averaged ) :-
     kv_average_sums_v_given_k( T, K, 1, V, Rem, HowMany, Sum ),
     Av is Sum / HowMany,
     Averaged = [K-Av|TAveraged],
     kv_average_v_for_k( Rem, TAveraged ).

kv_average_sums_v_given_k( [], _K, Count, Acc, [], Count, Acc ).
kv_average_sums_v_given_k( [K-V|T], K, Count, Acc, Rem, HowMany, Sum ) :-
     !,
     NxtAcc is Acc + V,
     NxtCount is Count + 1,
     kv_average_sums_v_given_k( T, K, NxtCount, NxtAcc, Rem, HowMany, Sum ).
kv_average_sums_v_given_k( L, _K, Count, Sum, L, Count, Sum ).
