:- ensure_loaded( library(system) ).		% datime/1.
:- ensure_loaded( '$HOME/islp/mcmcms/src/init_lib' ).
:- ensure_loaded( '$HOME/islp/mcmcms/bns/src/llhood/jc_llhood_ratio_fa_dstr' ).
:- ensure_loaded( library(pbc_true) ). 	%/1.
:- ensure_loaded( library(gauge) ). 	%view/1.

% test :- test_mem_vs_fall.
test :- test_integrity.

% memory
% 12:08:49 - 12:14:57
/* 
memory (total)       7065568 bytes
   program space     1619984 bytes:    1455064 in use,    164920 free
   global stack      5385076 bytes:    2885300 in use,   2499776 free
   local stack         16724 bytes:        308 in use,     16416 free
   trail stack         21808 bytes:         48 in use,     21760 free
   control stack       21976 bytes:        216 in use,     21760 free
   atoms              134791 bytes:       4185 in use,    257958 free
       0.040 sec. for 12 global, 0 local, and 34 control space overflows
      63.280 sec. for 6080 garbage collections which collected 1033069368 bytes
       0.000 sec. for 0 atom garbage collections which collected 0 atoms 0 bytes
     305.820 sec. runtime
     412.850 sec. elapsed time
*/

test_mem_vs_fall :-
	get_n_families( 100, 38, HndFms ),
	statistics,
	test_n_fall( HndFms ),
	% test_n_mem( HndFms ),
	statistics,
	view( user:_ ).
	% test_mem( [2,6,12,18,22], 25 ).
	% test_fall( [2,6,12,18,22], 25 ).

test_integrity :-
	get_n_families( 10, 38, HndFms ),
	read_terms_to_list( '/local/d0p6/local/nicos/jmlr_20041231/data/alarm_10K.pl', Data ), 
	compile( '/local/d0p6/local/nicos/jmlr_20041231/data/alarm_10K' ),
	tell( malaka ),
	test_n_integrity( HndFms, 1, Data ),
	told.

test_n_integrity( [], _N, _Data ).
test_n_integrity( [Ch-Pars|T], N, Data ) :-
	test_mem( Pars, Ch, Data, MemInst ),
	sort( MemInst, SortMem ),
	test_fall( Pars, Ch, FallInst ),
	sort( FallInst, SortFall ),
	( SortMem == SortFall->
		write( aggreement(N) ), nl
		;
		write( difference(N)/MemInst-FallInst ), nl
	),
	NxN is N + 1,
	test_n_integrity( T, NxN, Data ).

test_n_mem( HndFms ) :-
	read_terms_to_list( '/local/d0p6/local/nicos/jmlr_20041231/data/alarm_10K.pl', Data ), 
	write( test_n_mem ), nl,
	test_n_mem_1( HndFms, Data ).
	
test_n_mem_1( [], _Data ).
test_n_mem_1( [Ch-Pars|T], Data ) :-
	test_mem( Pars, Ch, Data, _ ),
	test_n_mem_1( T, Data ).

test_n_fall( Fms ) :-
	compile( '/local/d0p6/local/nicos/jmlr_20041231/data/alarm_10K' ),
	write( test_fall ), nl,
	test_n_fall_1( Fms ).

test_n_fall_1( [] ).
test_n_fall_1( [Ch-Pars|T] ) :-
	test_fall( Pars, Ch, _ ),
	test_n_fall_1( T ).

get_n_families( Lim, Nodes, Fms ) :-
	0 < Lim,
	!,
	random( 1, Nodes, Child ),
	First is Child - 1,
	select_or_parents_for( First, Parents ),
	Fms = [Child-Parents|TFms],
	RedLim is Lim - 1,
	get_n_families( RedLim, Nodes, TFms ).
get_n_families( _Lim, _Nodes, [] ).

select_or_parents_for( 0, [] ) :- !.
select_or_parents_for( CandP, Parents ) :-
	( pbc_true(0.5) ->
		Parents = [CandP|TPs]
		;
		TPs = Parents
	),
	NxCP is CandP - 1,
	select_or_parents_for( NxCP, TPs ).

test_fall( PIds, ChId, SortedInstances ) :-
	datime( Start ), write( start_at(Start) ), nl,
	kv_compose( [ChId|PIds], [ChildVar|PVars], Comp ),
	keysort( Comp, SortOfInterest ),
	pairs_with_var_list( SortOfInterest, 1, ArgsList ),
	Goal =.. [data,ArgsList,Tms],
	% Goal = data( List, Tms ),
	% List = [_,_,_,_,_,_,_,_,_,_,
			% _,_,_,_,_,_,_,_,_,_,
			% _,_,_,_,_,_,_,_,_,_,
			% _,_,_,_,_,_,_], 
	% bind([ChId|PIds],List,[ChildVar|PVars]),
	findall( PVars-(ChildVar,Tms), Goal, Instances ),
	keysort( Instances, SortedInstances ),
	datime( End ), write( ended_at(End) ), nl.
	% length( SortedInstances, Lgth ), 
	% write( length(Lgth) ), nl.

test_mem( PIds, ChId, Data, SortedInstances ) :-
	datime( Start ), write( start_at(Start) ), nl,
	data_instances( Data, PIds, ChId, [], SortedInstances ),
	datime( End ), write( ended_at(End) ), nl.
	% length( SortedInstances, Lgth ), 
	% write( length(Lgth) ), nl.

test_pick :- true.

read_terms_to_list( File, Data ) :-
	open( File, read, R ),
	read( R, Term ),
	read_terms_to_list_1( Term, R, Data ).

read_terms_to_list_1( end_of_file, _R, [] ) :- !.
read_terms_to_list_1( data(List,Tms), R,  [Datum-Tms|T] ) :-
	Datum =.. [data|List],
	read( R, NxTerm ),
	read_terms_to_list_1( NxTerm, R, T ).
