:- op( 600, xfy, ~ ).
:- ensure_loaded( n_samples ).		%.st
:- ensure_loaded( data_to_cart_root ).

:- ensure_loaded( '../bns/auxil/disp_bn' ).
:- ensure_loaded( library(charsio) ).		% write_to_chars/2
:- ensure_loaded( cart_llhood ).
:- ensure_loaded( test_dtcp ).
:- ensure_loaded( split_cart_to_model ).
:- ensure_loaded( '../src/lib/delete_file_if' ).

unify :-
	n_samples( [stem(unifyn),samples(100),dot_ith_intervals(10),
				precall(test_dtcp(ADIds,RootPs)),
				call(split(leaf(0,ADIds,RootPs),0.5/0.5,Tree)-Tree),
				slp(unify), post(cart_to_structure)
				],
			Tmp, Frqs ),
	% do_something_latex_with_tmp(Tmp)
	For = ' 100 samples of split(leaf(0,ADIds,RootPs),0.5/0.5,Tree)-Tree ',
	eps_n_latex_graph_freqs( Frqs, unifyn, For, 1, 'unify.slp', Tmp ),
	delete_file_if( Tmp ),
	display_carts( Frqs ).

cart_to_structure( Cart, (Str,Lhood) ) :-
	cart_to_structure( Cart, 1, _, _, Str ),
	split_cart_to_model( Cart, Model ),
	cart_llhood( Model, Lhood ).

cart_to_structure( cart(F,S,L,R), N, N:F:S, N3, [L1-N,R1-N,N:F:S|T] ) :-
	% write_to_chars( N:F:S, FSCs ),
	% cart_labls_to_node( N, F, S, FS ),
	% append( FSCs, [0''], FSLQ ),
	% atom_codes( FS, [0''|FSLQ] ),
	write( cart(F,S) ), nl,
	N1 is N + 1,
	cart_to_structure( L, N1, L1, N2, LStr ),
	cart_to_structure( R, N2, R1, N3, RStr ),
	append( LStr, RStr, T ).
cart_to_structure( leaf(S_D,S_Ids,_P), N, N, NxN, [N] ) :-
	write( leaf(S_D, S_Ids) ), nl,
	NxN is N + 1.
	% write_to_chars( D:P, DPCs ),
	% atom_codes( DP, DPCs ).

cart_labls_to_node( N, F, S, FS ) :-
	number_codes( N, NCs ),
	number_codes( F, FCs ),
	write_to_chars( S, SCs ),
	append( [0'_|FCs], [0'_|SCs], SFxs ),
	append( NCs, SFxs, FSCs ),
	atom_codes( FS, FSCs ).

display_carts( [] ).
display_carts( [(H,LH)-F|T] ) :-
	% display_cart( H ),
	mktemp( tmp_disp_XXXXXX, Tmp ),
	cart_str_to_disp_dag( H, LH, F, Tmp ),
	atom_concat( Tmp, ' &', TmpSfx ),
	atom_concat( 'doted ', TmpSfx, Doted ),
	shell( Doted ),
	sleep( 1 ),
	% disp_bn( HBN ),
	delete_file_if( Tmp ),
	display_carts( T ).
	% cart_str_to_disp_dag( [2-1,(3:1:[a,b])-1,1:1:[b,c],2,4-3,5-3,3:1:[a,b],4,5], test_dag ).
	% cart_str_to_disp_dag( [(2:2:5)-1,(3:2:5)-1,1:1:[a,b,c],3-2,(4:2:8)-2,2:2:5,3,5-4,6-4,4:2:8,5,6,4-3,5-3,3:2:5,4,5], test_dag ).

cart_str_to_disp_dag( Str, Lk, Frq, Dag ) :-
	pick_parents( Str, Parents, Leaves, Pairs, Edges ),
	keysort( Pairs, Sorted ),
	edges_parents_to_dag( Parents, Leaves, Sorted, Edges, Lk, Frq, Dag ).

% edges_parents_to_dag( [Left,Right|T], Parents, Dag ) :-
edges_parents_to_dag( Parents, Leaves, Pairs, Edges, Lk, Frq, Dag ) :-
	open( Dag, write, S ),
	write( S, 'digraph G {' ), nl( S ),
	write( S, '  node [style=filled]' ), nl( S ),
	write( S, '  node [shape=egg]' ), nl( S ),
	write( S, '  edge [color=red]' ), nl( S ),
	% write( S, '  1 [color=red]' ), nl( S ),
	leave_parent_nodes( Pairs, S ),
	% leave_nodes( Leaves, S ),
	% parent_nodes( Parents, S ),
	edges_w_labels( Edges, Parents, Leaves, S ),
	write( S, '"' ), write( S, Lk ), write( S, ' \\n ' ),
	write( S, Frq ), write( S, '"' ),
	write( S, '[color=lemonchiffon2, shape=invhouse]' ), nl( S ),
	write( S, '}' ),
	close( S ).

leave_parent_nodes( [], _S ).
leave_parent_nodes( [Id-Node|T], S ) :-
	( Id == Node ->
		write( S, '  ' ), write( S, Node ), 
		write( S, ' [color=green]' ), nl( S )
		;
		( Id =:= 1 ->
			write( S, '  ' ), write( S, Node ),
			write( S, ' [color=red]' ), nl( S )
			;
			write( S, '  ' ), write( S, Node ), 
			write( S, ' [color=orange]' ), nl( S )
		)
	),
	leave_parent_nodes( T, S ).
	

parent_nodes( [], _S ).
parent_nodes( [N-LLbl|T], S ) :-
	( N =:= 1 ->
		write( S, '  ' ), write( S, LLbl ),
		write( S, ' [color=red]' ), nl( S )
		;
		write( S, '  ' ), write( S, LLbl ), 
		write( S, ' [color=orange]' ), nl( S )
	),
	parent_nodes( T, S ).

leave_nodes( [], _S ).
leave_nodes( [H|T], S ) :-
	write( S, '  ' ), write( S, H ), 
	write( S, ' [color=green]' ), nl( S ),
	leave_nodes( T, S ).

edges_w_labels( [], _Parents, _Leaves, _S ).
edges_w_labels( [LCh-Par,RCh-Par|T], Parents, Leaves, S ) :-
	memberchk( Par-ParLbl, Parents ),
	write( S, '  ' ), write( S, ParLbl ),
	% ( nth( NL, Leaves, LCh ) -> true ; NL = LCh ),
	( cart_parent( LCh, _-PrpLCh ) -> true ; PrpLCh = LCh ),
	write( S, ' -> ' ), write( S, PrpLCh ), nl( S ),
	% write( S, ' [label=' ), 
	% select( Par-LLbl, Parents, NxParents ),
	% write( S, LLbl ), write( S, ']' ), nl( S ),
	write( S, '  ' ), write( S, ParLbl ),
	% ( nth( NR, Leaves, RCh ) -> true ; NR = RCh ),
	( cart_parent( RCh, _-PrpRCh ) -> true ; PrpRCh = RCh ),
	write( S, ' -> ' ), write( S, PrpRCh ), nl( S ),
	edges_w_labels( T, Parents, Leaves, S ).

pick_parents( [], [], [], [], [] ).
pick_parents( [H|T], Par, Leaves, Pairs, Edges ) :-
	( cart_edge(H,HEdge) ->
		Par = TPar,
		Leaves = TLeaves,
		Pairs = TPairs,
		Edges = [HEdge|TEdges]
		;
		( cart_parent(H,ParH) ->
			Par = [ParH|TPar],
			Leaves = TLeaves,
			Pairs = [ParH|TPairs],
			Edges = TEdges
			;
			Par = TPar,
			Leaves = [H|TLeaves],
			Pairs = [H-H|TPairs],
			Edges = TEdges
		)
	),
	pick_parents( T, TPar, TLeaves,TPairs, TEdges ).

cart_edge( Child-Parent, Child-Parent ).
	% ( cart_parent( Child, _N-EdgeCh ) -> true
		% ; EdgeCh = Child ),
	% ( cart_parent( Parent, _N-EdgePar ) -> true
		% ; EdgePar = Parent ).

cart_parent( N:F:Ch, N-ParH ) :-
	% number_codes( N, Ns ), 
	% number_codes( F, Fs ),
	write_to_chars( N:F~Ch, ChCs ),
	append( [0'"|ChCs], [0'"], ParHCs ),
	% append( [0'_|Fs], [0'_|ChCs], SfxCs ),
	% append( Ns, SfxCs, ParHCs ),
	atom_codes( ParH, ParHCs ).
