:- module( introduces, 
			[
				edge_introduces_n_inbreedings/5,
				edge_introduces_n_promiscuities/5,
				edge_introduces_generation/4,
				max_length_path_from/3 ] ).

:- compile( sicstus_path ).

:- ensure_loaded( library(lists) ).		% member/2, memberchk/2.
:- ensure_loaded( library(ugraphs) ).		% reachable/3, min_path/5.
:- ensure_loaded( library(ordsets) ).		% ord_intersect/2
:- ensure_loaded( '../../../src/init_lib' ).
:- ensure_loaded( library(memberchk_kv_or_empty) ). % /3.

/*
edge_introduces_n_inbreedings( Person, Parent, Ped, N ) :-
	Ped = ped(AncG,DscG,Prtns,_Mtrcs), 
	reachable( Person, DscG, AddPsDscs ),
	ord_del_element( AddPsDscs, Person, PsDscs ),
	findall( PromDsc, 
			(
				member(PromDsc,PsDscs),
				partner_of(PromDsc,Prtns,DscPartner),
				min_path_pl(DscPartner,Parent,AncG,_MnPath,_MinCost ),
				\+ incestious_pair( PromDsc, DscPartner, AncG )
			), PromDscs ),
	length( PromDscs, N ).
	*/
	% 2005ac21
edge_introduces_n_inbreedings( P, Parent, Ped, Inb, NewICh ) :-
	Ped = ped(AncG,DscG,_Prtns,ICh,_Mtrcs), 
	reachable( Parent, AncG, ParentAnc ),
	reachable( P, DscG, PDsc ),
	findall( InbredP, 
				( member( Anc, ParentAnc ),
				member( InbredP, PDsc ),
				min_path_pl(InbredP,Anc,AncG,_MnPath,_MinCost ),
				% MinCost > 1,
				% \+ ( Anc == Parent, InbredP == P ),
				\+ ord_member( InbredP, ICh ) ),
			NewInbreds
		),
	list_to_ord_set( NewInbreds, NewICh ),
	length( NewICh, Inb ).

% fathers cannt introduce promiscuities, because they are considered first.
edge_introduces_n_promiscuities( male, _Person, _SParent, _Ped, 0 ) :- !.
edge_introduces_n_promiscuities( female, Person, SParent, Ped, NofPromPrs ) :-
	Ped = ped( AncG, DscG, _, _, _ ),
	edge_introduces_n_promiscuities_1( Person, SParent, AncG, DscG, NofPromPrs ).
edge_introduces_n_promiscuities_1( Person, SParent, AncG, DscG, NofPromPrs ) :-
	memberchk_kv_or_empty( DscG, SParent, SPaChn ),
	memberchk_kv_or_empty( AncG, Person, PersonEPa ),
	existing_parents_children( PersonEPa, DscG, EPaChn ),
	ord_add_element( SPaChn, Person, AllSPaChn ),
	% ord_symdiff( SPaChn, EPaChn, PromChn ), 
	ord_subtract( SPaChn, EPaChn, DiffSPa ),
	ord_subtract( EPaChn, AllSPaChn, DiffEPa ),
	% length( PromChn, NumOfPromChn ), 
	length( DiffSPa, NofNewSPa ),
	length( DiffEPa, NofNewEPa ),
	NofPromPrs is NofNewSPa + NofNewEPa.
	% NofPromPrs is integer(( (NumOfPromChn * (NumOfPromChn - 1) ) ) / 2).
	% only correct when EPaChn have both parents set. currently, this holds.

	% memberchk( Parent-OthChildren, DscG ), 
	% select both parents in one step ? maybe better
	% length( OthChildren, N ).
% edge_introduces_n_promiscuities( _Person, _Parent, _DscG, 0).
	% if memberchk/2, above, fails then we have a new unkown person,
	% in which case, there are no promiscuities introduced.

existing_parents_children( [], _Gr,  [] ).
existing_parents_children( [H], Gr, HChn ) :-
	memberchk_kv_or_empty( Gr, H, HChn ).
% existing_parents_children( [F,S|T], _ ) :- this is an error...

partner_of( Person, Partners, Partner ) :-
	memberchk( Person-PersonPartners, Partners ),
	member( Partner, PersonPartners ).

incestious_pair( PersonA, PersonB, AncG ) :-
	reachable( PersonA, AncG, ReachedByA ),
	reachable( PersonB, AncG, ReachedByB ),
	ord_intersect( ReachedByA, ReachedByB ).

max_length_path_from( Person, Ped, Max ) :-
	Ped = ped(Graph,_,_,_,_),
	findall( Length, 
			( 
			  path_pl(Person,Graph,Path), 
			  length(Path,Length)
			),
			  		Lengths ),
	max_list_w_empty( Lengths, Max ).

max_list_w_empty( [], Max ) :- 
	!, Max is 0.
max_list_w_empty( List, Max ) :- 
	max_list( List, MaxPlus ), 
	Max is MaxPlus - 1.
