function input_posts=compute_posterior_over_predictors(D,samples,varargin)
% input_posteriors=compute_posterior_over_predictors(D,samples,OPTIONS)
% Computes posterior probabilities over individual predictors, as described
% in Eq. 11 of manuscript by Mukherjee, Pelech, Neve, Spellman, Gray and Speed, 
% submitted to Bioinformatics, 2008
% 
% Output:
% input_posteriors - row vector containing posterior probabilities that
%   individual predictors form part of underlying model (see Eq. 11 of
%   paper for details). input_posteriors(j) is probability for j^th
%   predictor
% 
% Inputs:
% D - binary data with observations in columns, and dimensions in rows,
%   plus the last row containing responses Y; matrix is thus of size (p+1) by n
% samples - MCMC samples produced using sparse_combinatorial_inference.m
% 
% OPTIONS:
% 'burn_in' - MCMC burn in, set to T/10 by default
% 
% Example:
% posteriors=compute_posterior_over_predictors(D,samples,'burn_in',burn_in)


T=length(samples.LP);

% DEFAULT burn_in
burn_in=ceil(T/10);


args = varargin;
nargs = length(args);
for i=1:2:nargs
 switch args{i},
  case 'burn_in',  burn_in = args{i+1};
     otherwise disp('Unknown option.');disp(args{i}); return;
 end
end

n_inputs=size(D,1)-1;

input_posts=zeros(1,n_inputs);

for i=burn_in+1:T
   inputs_t=samples.inputs{i};
   input_posts(inputs_t)=input_posts(inputs_t)+1;
end

input_posts=input_posts./(T-burn_in);